/**
 * Sound (not music) management

 * Copyright (C) 1997, 1998, 1999, 2002, 2003  Seth A. Robinson
 * Copyright (C) 2003  Shawn Betts
 * Copyright (C) 2005, 2007  Sylvain Beucler
 * Copyright (C) 2023-2024 Yeoldetoast

 * This file is part of GNU FreeDink

 * GNU FreeDink is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.

 * GNU FreeDink is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#ifndef _SFX_H
#define _SFX_H

#include "SDL_mixer.h"
#include "soloud/soloud.h"
#include "soloud/soloud_wav.h"
#include "soloud/soloud_speech.h"

#define NUM_CHANNELS 64

extern /*bool*/ int sound_on;
extern int buf_samples, audio_samplerate;
//For MixerX settings from ini
extern int adlemu, opnemu;
extern char soundfont[200];
//sfx
extern SoLoud::Soloud gSoloud;
extern SoLoud::Wav gWave;
extern SoLoud::Speech speech;
extern SoLoud::Bus gBus;

extern void sfx_init();
extern void sfx_quit();
extern int CreateBufferFromWaveFile(char* filename, int dwBuf);
extern void EditorSoundPlayEffect(int sound);
extern int SoundPlayEffect(int sound, int min, int plus, int sound3d,
						/*bool*/ int repeat);
extern int playing(int sound);
extern int SoundStopEffect(int sound);
extern void kill_repeat_sounds(void);
extern void kill_repeat_sounds_all(void);
extern void sfx_log_meminfo(void);
extern void update_sound(void);
extern void halt_all_sounds();

/* DinkC procedures */
extern int playsound(int sound, int min, int plus, int sound3d, int repeat);
extern void sound_set_kill(int soundbank);
extern void sound_set_survive(int soundbank, int survive);
extern void sound_set_vol(int soundbank, int volume);
extern int playsfx(char* filename, int speed, int pan);

//Lua sound properties
extern void lua_set_vol(int soundbank, int volume);
extern int sound_get_vol(int soundbank);
extern bool sound_get_survive(int soundbank);
extern void sound_set_pan(int soundbank, int pan);
extern int sound_get_pan(int soundbank);
extern void sound_set_hz(int soundbank, int hz);
extern int sound_get_hz(int soundbank);
extern int sound_get_speed(int soundbank);
extern void sound_set_speed(int soundbank, int speed);
extern void sound_set_pause(int soundbank, bool paused);
extern bool sound_get_pause(int soundbank);
extern void sound_set_loop_point(int soundbank, int looppoint);
extern int sound_get_loop_point(int soundbank);
extern int sound_get_loop_count(int soundbank);
extern int sound_get_pos(int soundbank);
extern void sound_set_looping(int soundbank, bool repeat);
extern bool sound_get_looping(int soundbank);
//Extra Lua functions for sounds
extern void sound_fade_vol(int soundbank, int vol, int time);
extern void sound_fade_pan(int soundbank, int pan, int time);
extern void sound_fade_speed(int soundbank, int speed, int time);
extern void sound_schedule_pause(int soundbank, int time);
extern void sound_schedule_stop(int soundbank, int time);
extern void sound_oscillate_vol(int soundbank, int from, int to, int time);
extern void sound_oscillate_pan(int soundbank, int from, int to, int time);
extern void sound_oscillate_speed(int soundbank, int from, int to, int time);

//sfx 3d sound proeprties
extern float atten;
extern float dist_min;
extern float dist_max;
extern int dist_model;

//test, sounds horrid
extern void play_midi_as_sfx(char* file, char* sf2);

#endif
